@@ -26,15 +26,23 @@
 #include "babl-db.h"
 #include "babl-ref-pixels.h"
 
-static Babl *
-conversion_new (const char    *name,
-                int            id,
-                Babl          *source,
-                Babl          *destination,
-                BablFuncLinear linear,
-                BablFuncPlane  plane,
-                BablFuncPlanar planar,
-                void          *user_data)
+static int model_is_rgba (const Babl *model)
+{
+  const Babl *RGBA = babl_model_from_id (BABL_RGBA);
+  if (model == RGBA || model->model.data == RGBA)
+    return 1;
+  return 0;
+}
+
+Babl *
+_conversion_new (const char    *name,
+                 int            id,
+                 const Babl    *source,
+                 const Babl    *destination,
+                 BablFuncLinear linear,
+                 BablFuncPlane  plane,
+                 BablFuncPlanar planar,
+                 void          *user_data)
 {
   Babl *babl = NULL;
 
@@ -112,19 +120,16 @@ conversion_new (const char    *name,
     {
       const Babl *src_format = NULL;
       const Babl *dst_format = NULL;
-      if (BABL (babl->conversion.source) == babl_model_from_id (BABL_RGBA))
-        {
-          src_format = babl_format_from_id (BABL_RGBA_DOUBLE);
-          dst_format = babl_format_with_model_as_type (
-            BABL (babl->conversion.destination),
-            babl_type_from_id (BABL_DOUBLE));
-        }
-      else if (BABL (babl->conversion.destination) == babl_model_from_id (BABL_RGBA))
+
+      if (model_is_rgba (BABL (babl->conversion.source)) ||
+          model_is_rgba (BABL (babl->conversion.destination)))
         {
           src_format = babl_format_with_model_as_type (
             BABL (babl->conversion.source),
             babl_type_from_id (BABL_DOUBLE));
-          dst_format = babl_format_from_id (BABL_RGBA_DOUBLE);
+          dst_format = babl_format_with_model_as_type (
+            BABL (babl->conversion.destination),
+            babl_type_from_id (BABL_DOUBLE));
         }
       else
         {
@@ -161,16 +166,40 @@ create_name (Babl *source, Babl *destination, int type)
     }
   else
     {
-      snprintf (buf, 512 - 1, "%s %s to %s",
+      snprintf (buf, 512 - 1, "%s %s to %s %i",
                 type == BABL_CONVERSION_LINEAR ? "" :
                 type == BABL_CONVERSION_PLANE ? "plane " :
                 type == BABL_CONVERSION_PLANAR ? "planar " : "Eeeek! ",
                 source->instance.name,
-                destination->instance.name);
+                destination->instance.name,
+                collisions);
       buf[511] = '\0';
     }
   return buf;
 }
+const char *
+babl_conversion_create_name (Babl *source, Babl *destination, int type);
+
+const char *
+babl_conversion_create_name (Babl *source, Babl *destination, int type)
+{
+  Babl *babl;
+  char *name;
+  int id = 0;
+  collisions = 0;
+  name = create_name (source, destination, type);
+  babl = babl_db_exist (db, id, name);
+  while (babl)
+    {
+      /* we allow multiple conversions to be registered per extender, each
+         of them ending up with their own unique name
+       */
+      collisions++;
+      name = create_name (source, destination, type);
+      babl = babl_db_exist (db, id, name);
+    }
+  return name;
+}
 
 const Babl *
 babl_conversion_new (const void *first_arg,
@@ -267,21 +296,9 @@ babl_conversion_new (const void *first_arg,
       type = BABL_CONVERSION_PLANAR;
     }
 
-  collisions = 0;
-  name = create_name (source, destination, type);
+  name = (void*) babl_conversion_create_name (source, destination, type);
 
-  babl = babl_db_exist (db, id, name);
-  while (babl)
-    {
-      /* we allow multiple conversions to be registered per extender, each
-         of them ending up with their own unique name
-       */
-      collisions++;
-      name = create_name (source, destination, type);
-      babl = babl_db_exist (db, id, name);
-    }
-
-  babl = conversion_new (name, id, source, destination, linear, plane, planar, 
+  babl = _conversion_new (name, id, source, destination, linear, plane, planar,
                          user_data);
 
   /* Since there is not an already registered instance by the required
@@ -294,16 +311,16 @@ babl_conversion_new (const void *first_arg,
   return babl;
 }
 
-static long
+static void
 babl_conversion_linear_process (BablConversion *conversion,
                                 const void     *source,
                                 void           *destination,
                                 long            n)
 {
-  return conversion->function.linear (source, destination, n, conversion->data);
+  conversion->function.linear ((void*)conversion, source, destination, n, conversion->data);
 }
 
-static long
+static void
 babl_conversion_plane_process (BablConversion *conversion,
                                const void     *source,
                                void           *destination,
@@ -311,13 +328,13 @@ babl_conversion_plane_process (BablConversion *conversion,
                                int             dst_pitch,
                                long            n)
 {
-  return conversion->function.plane (source, destination,
-                                     src_pitch, dst_pitch,
-                                     n,
-                                     conversion->data);
+  conversion->function.plane ((void*)conversion, source, destination,
+                              src_pitch, dst_pitch,
+                              n,
+                              conversion->data);
 }
 
-static long
+static void
 babl_conversion_planar_process (BablConversion *conversion,
                                 BablImage      *source,
                                 BablImage      *destination,
@@ -333,8 +350,8 @@ babl_conversion_planar_process (BablConversion *conversion,
 
   memcpy (src_data, source->data, sizeof (void *) * source->components);
   memcpy (dst_data, destination->data, sizeof (void *) * destination->components);
-
-  return conversion->function.planar (source->components,
+  conversion->function.planar ((void*)conversion,
+                                      source->components,
                                       src_data,
                                       source->pitch,
                                       destination->components,
@@ -446,14 +463,8 @@ babl_conversion_error (BablConversion *conversion)
   Babl *fmt_source;
   Babl *fmt_destination;
 
-  const Babl *fmt_rgba_double = babl_format_new (babl_model ("RGBA"),
-                                                 babl_type ("double"),
-                                                 babl_component ("R"),
-                                                 babl_component ("G"),
-                                                 babl_component ("B"),
-                                                 babl_component ("A"),
-                                                 NULL);
-
+  const Babl *fmt_rgba_double = babl_format_with_space ("RGBA double",
+                                                 conversion->destination->format.space);
   double  error       = 0.0;
   long    ticks_start = 0;
   long    ticks_end   = 0;
@@ -499,7 +510,7 @@ babl_conversion_error (BablConversion *conversion)
         fmt_source->class_type == BABL_FORMAT &&
         fmt_destination->class_type == BABL_FORMAT))
     {
-      conversion->error = 0.000042;
+      conversion->error = 0.0000042;
     }
 
   source                      = babl_calloc (test_pixels, fmt_source->format.bytes_per_pixel);
@@ -548,4 +559,14 @@ babl_conversion_error (BablConversion *conversion)
   return error;
 }
 
+const Babl *babl_conversion_get_source_space      (const Babl *conversion)
+{
+  return conversion->conversion.source->format.space;
+}
+
+const Babl *babl_conversion_get_destination_space (const Babl *conversion)
+{
+  return conversion->conversion.destination->format.space;
+}
+
 BABL_CLASS_IMPLEMENT (conversion)
