@@ -21,7 +21,7 @@
 #include "babl-internal.h"
 #include "babl-ref-pixels.h"
 
-#define BABL_TOLERANCE             0.000001
+#define BABL_TOLERANCE             0.000005
 #define BABL_MAX_COST_VALUE        2000000
 #define BABL_HARD_MAX_PATH_LENGTH  8
 #define BABL_MAX_NAME_LEN          1024
@@ -31,7 +31,7 @@
 #endif
 
 #define NUM_TEST_PIXELS            (babl_get_num_path_test_pixels ())
-#define MAX_BUFFER_SIZE            2048 /* XXX: reasonable size for this should be profiled */
+#define MAX_BUFFER_SIZE            512
 
 
 int   babl_in_fish_path = 0;
@@ -84,9 +84,10 @@ process_conversion_path (BablList   *path,
 
 static void
 get_conversion_path (PathContext *pc,
-                     Babl *current_format,
-                     int current_length,
-                     int max_length);
+                     Babl        *current_format,
+                     int          current_length,
+                     int          max_length,
+                     double       legal_error);
 
 char *
 _babl_fish_create_name (char       *buf,
@@ -134,7 +135,7 @@ static int max_path_length (void)
   if (env)
     max_length = atoi (env);
   else
-    max_length = 3; /* reducing this number makes finding short fishes much
+    max_length = 4; /* reducing this number makes finding short fishes much
                        faster - even if we lose out on some of the fast
                        bigger fish
                      */
@@ -150,6 +151,32 @@ int _babl_max_path_len (void)
   return max_path_length ();
 }
 
+static int
+bad_idea (const Babl *from, const Babl *to, const Babl *format)
+{
+  if (babl_format_has_alpha (from) &&
+      babl_format_has_alpha (to) &&
+      !babl_format_has_alpha (format))
+  {
+    return 1;
+  }
+  if (from->format.components > format->format.components &&
+      to->format.components > format->format.components)
+  {
+    return 1;
+  }
+  if (from->format.type[0]->bits > format->format.type[0]->bits &&
+      to->format.type[0]->bits > format->format.type[0]->bits)
+  {
+    /* XXX: perhaps we especially avoid going to half-float, when
+     * going between u16 formats as well? */
+    return 1;
+  }
+
+  return 0;
+}
+
+
 /* The task of BablFishPath construction is to compute
  * the shortest path in a graph where formats are the vertices
  * and conversions are the edges. However, there is an additional
@@ -163,12 +190,12 @@ int _babl_max_path_len (void)
  * implemented by recursive function get_conversion_path ().
  */
 
-
 static void
 get_conversion_path (PathContext *pc,
                      Babl        *current_format,
                      int          current_length,
-                     int          max_length)
+                     int          max_length,
+                     double       legal_error)
 {
   if (current_length > max_length)
     {
@@ -183,16 +210,17 @@ get_conversion_path (PathContext *pc,
       double path_cost  = 0.0;
       double ref_cost   = 0.0;
       double path_error = 1.0;
+#if 0
       int    i;
-
       for (i = 0; i < babl_list_size (pc->current_path); i++)
         {
           path_error *= (1.0 + babl_conversion_error ((BablConversion *) pc->current_path->items[i]));
         }
 
-      if (path_error - 1.0 <= _babl_legal_error ())
+      if (path_error - 1.0 <= legal_error )
                 /* check this before the more accurate measurement of error -
                    to bail earlier */
+#endif
         {
           FishPathInstrumentation fpi;
           memset (&fpi, 0, sizeof (fpi));
@@ -202,16 +230,13 @@ get_conversion_path (PathContext *pc,
 
           get_path_instrumentation (&fpi, pc->current_path, &path_cost, &ref_cost, &path_error);
           if(debug_conversions && current_length == 1)
-            fprintf (stderr, "%s  error:%f cost:%f  \n", 
-                 babl_get_name (pc->current_path->items[0]),
-                 /*babl_get_name (pc->fish_path->fish.source),
-                 babl_get_name (pc->fish_path->fish.destination),*/
-                 path_error,
-                 path_cost /*, current_length*/);
+            fprintf (stderr, "%s  error:%f cost:%f  \n",
+                 babl_get_name (pc->current_path->items[0]), path_error, path_cost);
 
           if ((path_cost < ref_cost) && /* do not use paths that took longer to compute than reference */
-              (path_cost < pc->fish_path->fish_path.cost) &&
-              (path_error <= _babl_legal_error ()))
+              (path_cost < pc->fish_path->fish_path.cost) && // best thus far
+              (path_error <= legal_error )               // within tolerance
+              )
             {
               /* We have found the best path so far,
                * let's copy it into our new fish */
@@ -227,7 +252,7 @@ get_conversion_path (PathContext *pc,
   else
     {
       /*
-       * Bummer, we have to search deeper... 
+       * we have to search deeper...
        */
       BablList *list;
       int i;
@@ -243,11 +268,11 @@ get_conversion_path (PathContext *pc,
             {
               Babl *next_conversion = BABL (list->items[i]);
               Babl *next_format = BABL (next_conversion->conversion.destination);
-              if (!next_format->format.visited)
+              if (!next_format->format.visited && !bad_idea (current_format, pc->to_format, next_format))
                 {
                   /* next_format is not in the current path, we can pay a visit */
                   babl_list_insert_last (pc->current_path, next_conversion);
-                  get_conversion_path (pc, next_format, current_length + 1, max_length);
+                  get_conversion_path (pc, next_format, current_length + 1, max_length, legal_error);
                   babl_list_remove_last (pc->current_path);
                 }
             }
@@ -265,8 +290,8 @@ _babl_fish_create_name (char       *buf,
                         int         is_reference)
 {
   /* fish names are intentionally kept short */
-  snprintf (buf, BABL_MAX_NAME_LEN, "%s %p %p", "",
-            source, destination);
+  snprintf (buf, BABL_MAX_NAME_LEN, "%s %p %p %i", "",
+            source, destination, is_reference);
   return buf;
 }
 
@@ -283,16 +308,144 @@ _babl_fish_path_destroy (void *data)
   return 0;
 }
 
-Babl *
-babl_fish_path (const Babl *source,
-                const Babl *destination)
+static int
+show_item (Babl *babl,
+           void *user_data)
+{
+  BablConversion *conv = (void *)babl;
+
+  if (conv->destination->class_type == BABL_FORMAT)
+  {
+    fprintf (stderr, "%s : %.12f\n", babl_get_name (babl), babl_conversion_error(conv));
+  }
+
+  return 0;
+}
+
+static int
+alias_conversion (Babl *babl,
+                  void *user_data)
+{
+  BablConversion *conv = (void *)babl;
+  BablSpace *space = user_data;
+
+  if ((conv->source->class_type == BABL_FORMAT) &&
+      (conv->destination->class_type == BABL_FORMAT))
+  {
+    if ((conv->source->format.space == (void*)babl_space ("sRGB")) &&
+        (conv->destination->format.space == babl_space ("sRGB")))
+  {
+    switch (conv->instance.class_type)
+    {
+      case BABL_CONVERSION_LINEAR:
+       babl_conversion_new (
+              babl_format_with_space (
+                    (void*)conv->source->instance.name, (void*)space),
+              babl_format_with_space (
+                    (void*)conv->destination->instance.name, (void*)space),
+              "linear", conv->function.linear,
+              "data", conv->data,
+              NULL);
+        break;
+      case BABL_CONVERSION_PLANAR:
+       babl_conversion_new (
+              babl_format_with_space (
+                    (void*)conv->source->instance.name, (void*)space),
+              babl_format_with_space (
+                    (void*)conv->destination->instance.name, (void*)space),
+              "planar", conv->function.planar,
+              "data", conv->data,
+              NULL);
+        break;
+      case BABL_CONVERSION_PLANE:
+        babl_conversion_new (
+              babl_format_with_space (
+                    (void*)conv->source->instance.name, (void*)space),
+              babl_format_with_space (
+                    (void*)conv->destination->instance.name, (void*)space),
+              "plane", conv->function.plane,
+              "data", conv->data,
+              NULL);
+        break;
+      default:
+        break;
+    }
+  }
+  }
+  else
+  if ((conv->source->class_type == BABL_MODEL) &&
+      (conv->destination->class_type == BABL_MODEL))
+  {
+    if ((conv->source->model.space == (void*)babl_space ("sRGB")) &&
+        (conv->destination->model.space == babl_space ("sRGB")))
+  {
+    switch (conv->instance.class_type)
+    {
+      case BABL_CONVERSION_LINEAR:
+        babl_conversion_new (
+              babl_remodel_with_space (
+                    (void*)conv->source, (void*)space),
+              babl_remodel_with_space (
+                    (void*)conv->destination, (void*)space),
+              "linear", conv->function,
+              NULL);
+        break;
+      case BABL_CONVERSION_PLANAR:
+        babl_conversion_new (
+              babl_remodel_with_space (
+                    (void*)conv->source, (void*)space),
+              babl_remodel_with_space (
+                    (void*)conv->destination, (void*)space),
+              "planar", conv->function,
+              NULL);
+        break;
+      case BABL_CONVERSION_PLANE:
+        babl_conversion_new (
+              babl_remodel_with_space (
+                    (void*)conv->source, (void*)space),
+              babl_remodel_with_space (
+                    (void*)conv->destination, (void*)space),
+              "plane", conv->function,
+              NULL);
+        break;
+      default:
+        break;
+    }
+  }
+  }
+  else
+  if ((conv->source->class_type == BABL_TYPE) &&
+      (conv->destination->class_type == BABL_TYPE))
+  {
+  }
+  return 0;
+}
+
+
+static Babl *
+babl_fish_path2 (const Babl *source,
+                 const Babl *destination,
+                 double      tolerance)
 {
   Babl *babl = NULL;
+  const Babl *sRGB = babl_space ("sRGB");
   char name[BABL_MAX_NAME_LEN];
+  int is_fast = 0;
 
   _babl_fish_create_name (name, source, destination, 1);
   babl_mutex_lock (babl_format_mutex);
   babl = babl_db_exist_by_name (babl_fish_db (), name);
+
+  if (tolerance <= 0.0)
+  {
+    is_fast = 0;
+    tolerance = _babl_legal_error ();
+  }
+  else
+    is_fast = 1;
+
+  if (!is_fast)
+  {
   if (babl)
     {
       /* There is an instance already registered by the required name,
@@ -301,6 +454,43 @@ babl_fish_path (const Babl *source,
       babl_mutex_unlock (babl_format_mutex);
       return babl;
     }
+  }
+
+  if ((source->format.space != sRGB) ||
+      (destination->format.space != sRGB))
+  {
+    static const Babl *run_once[512]={NULL};
+    int i;
+    int done = 0;
+    for (i = 0; run_once[i]; i++)
+    {
+      if (run_once[i] == source->format.space)
+        done |= 1;
+      else if (run_once[i] == destination->format.space)
+        done |= 2;
+    }
+
+    if ((done & 1) == 0 && (source->format.space != sRGB))
+    {
+      run_once[i++] = source->format.space;
+      babl_conversion_class_for_each (alias_conversion, (void*)source->format.space);
+
+      _babl_space_add_universal_rgb (source->format.space);
+    }
+    if ((done & 2) == 0 && (destination->format.space != source->format.space) && (destination->format.space != sRGB))
+    {
+      run_once[i++] = destination->format.space;
+      babl_conversion_class_for_each (alias_conversion, (void*)destination->format.space);
+
+      _babl_space_add_universal_rgb (destination->format.space);
+    }
+
+    if (!done && 0)
+    {
+      babl_conversion_class_for_each (show_item, (void*)source->format.space);
+    }
+
+  }
 
   babl = babl_calloc (1, sizeof (BablFishPath) +
                       strlen (name) + 1);
@@ -331,12 +521,12 @@ babl_fish_path (const Babl *source,
      */
     babl_in_fish_path++;
 
-    get_conversion_path (&pc, (Babl *) source, 0, max_path_length ());
+    get_conversion_path (&pc, (Babl *) source, 0, max_path_length (), tolerance);
 
     /* second attempt,. at path length + 1*/
     if (babl->fish_path.conversion_list->count == 0 &&
         max_path_length () + 1 <= BABL_HARD_MAX_PATH_LENGTH)
-      get_conversion_path (&pc, (Babl *) source, 0, max_path_length () + 1);
+      get_conversion_path (&pc, (Babl *) source, 0, max_path_length () + 1, tolerance);
 
     babl_in_fish_path--;
     babl_free (pc.current_path);
@@ -364,7 +554,7 @@ babl_fish_path (const Babl *source,
 "programmatically by babl based on format description\n"
 "\n");
 
-        fprintf (stderr, "*WARNING*: missing babl fast path(s) between formats \"%s\" and \"%s\"\n",
+        fprintf (stderr, "*WARNING* missing babl fast path(s): \"%s\" to \"%s\"\n",
            babl_get_name (source),
            babl_get_name (destination));
 
@@ -375,11 +565,45 @@ babl_fish_path (const Babl *source,
   /* Since there is not an already registered instance by the required
    * name, inserting newly created class into database.
    */
-  babl_db_insert (babl_fish_db (), babl);
+  if (!is_fast)
+  {
+    babl_db_insert (babl_fish_db (), babl);
+  }
   babl_mutex_unlock (babl_format_mutex);
   return babl;
 }
 
+const Babl * babl_fast_fish (const void *source_format,
+                             const void *destination_format,
+                             const char *performance)
+{
+  double tolerance = 0.0;
+
+  if (!performance || !strcmp (performance, "default"))
+    tolerance = 0.0; // note: not _babl_legal_error() to trigger,
+                      // right code paths in babl_fish_path2
+  else if (!strcmp (performance, "exact"))
+    tolerance=0.0000000001;
+  else if (!strcmp (performance, "precise"))
+    tolerance=0.00001;
+  if (!strcmp (performance, "fast"))
+    tolerance=0.001;
+  else if (!strcmp (performance, "glitch"))
+    tolerance=0.01;
+  else {
+    tolerance = babl_parse_double (performance);
+  }
+
+  return babl_fish_path2 (source_format, destination_format, tolerance);
+}
+
+Babl *
+babl_fish_path (const Babl *source,
+                const Babl *destination)
+{
+  return babl_fish_path2 (source, destination, 0.0);
+}
+
 static long
 babl_fish_path_process (Babl       *babl,
                         const void *source,
@@ -543,9 +767,6 @@ process_conversion_path (BablList   *path,
           temp_buffer2 = align_16 (alloca (MIN(n, MAX_BUFFER_SIZE) * sizeof (double) * 5 + 16));
         }
 
-
-
-
       for (j = 0; j < n; j+= MAX_BUFFER_SIZE)
         {
           long c = MIN (n - j, MAX_BUFFER_SIZE);
@@ -600,14 +821,8 @@ init_path_instrumentation (FishPathInstrumentation *fpi,
 
   if (!fpi->fmt_rgba_double)
     {
-      fpi->fmt_rgba_double = babl_format_new (
-        babl_model ("RGBA"),
-        babl_type ("double"),
-        babl_component ("R"),
-        babl_component ("G"),
-        babl_component ("B"),
-        babl_component ("A"),
-        NULL);
+      fpi->fmt_rgba_double = babl_format_with_space ("RGBA double",
+                                                     fmt_destination->format.space);
     }
 
   fpi->num_test_pixels = babl_get_num_path_test_pixels ();
@@ -727,15 +942,5 @@ get_path_instrumentation (FishPathInstrumentation *fpi,
                                     fpi->ref_destination_rgba_double,
                                      fpi->num_test_pixels * 4);
 
-#if 0
-  fpi->fish_rgba_to_source->fish.processings--;
-  fpi->fish_reference->fish.processings--;
-  fpi->fish_destination_to_rgba->fish.processings -= 2;
-
-  fpi->fish_rgba_to_source->fish.pixels      -= fpi->num_test_pixels;
-  fpi->fish_reference->fish.pixels           -= fpi->num_test_pixels;
-  fpi->fish_destination_to_rgba->fish.pixels -= 2 * fpi->num_test_pixels;
-#endif
-
   *ref_cost = fpi->reference_cost;
 }
