@@ -112,16 +112,54 @@ match_conversion (Babl *conversion,
   return 0;
 }
 
+
+
 Babl *
 babl_conversion_find (const void *source,
                       const void *destination)
 {
   void *data = (void*)destination;
-
   babl_list_each (BABL (source)->type.from_list, match_conversion, &data);
-  if (data == (void*)destination) /* didn't change */
-    return NULL;
-  return data;
+  if (data != (void*)destination) /* didn't change */
+    return data;
+  data = NULL;
+
+  if (BABL (source)->class_type == BABL_MODEL)
+  {
+     const Babl *srgb_source = BABL (source)->model.data ? BABL (source)->model.data:source;
+     const Babl *srgb_destination = BABL (destination)->model.data ? BABL (destination)->model.data:destination;
+     Babl *reference = babl_conversion_find (srgb_source, srgb_destination);
+
+  /* when conversions are sought between models, with non-sRGB chromaticities,
+     we create the needed conversions from existing ones on the fly, and
+     register them. The conversions registered by the models should pick up the
+     RGB chromaticities, and TRC to use from the space on the model instead of
+     hard-coding it.
+   */
+
+     switch (reference->instance.class_type)
+     {
+        case BABL_CONVERSION_LINEAR:
+          return _conversion_new ("", 0, (void*)source, (void*)destination,
+                            reference->conversion.function.linear,
+                            NULL,
+                            NULL,
+                            reference->conversion.data);
+        case BABL_CONVERSION_PLANE:
+          return _conversion_new ("", 0, source, destination,
+                            NULL,
+                            reference->conversion.function.plane,
+                            NULL,
+                            reference->conversion.data);
+        case BABL_CONVERSION_PLANAR:
+          return _conversion_new ("", 0, source, destination,
+                            NULL,
+                            NULL,
+                            reference->conversion.function.planar,
+                            reference->conversion.data);
+     }
+  }
+  return NULL;
 }
 
 int
@@ -131,9 +169,9 @@ babl_fish_get_id (const Babl *source,
   /* value of 'id' will be used as argument for hash function,
    * substraction serves as simple combination of
    * source/destination values. */
-  ptrdiff_t id = source - destination;
+  int id = (((size_t)source * 93)) ^ ((size_t)destination);
   /* instances with id 0 won't be inserted into database */
-  id *= ((((size_t)  (source))) % 37);
+  id *= ((((size_t)  (destination))) % 37);
 
   if (id == 0)
     id = 1;
@@ -216,12 +254,12 @@ babl_fish (const void *source,
          * path.
          */
         babl_hash_table_find (id_htable, hashval, find_fish_path, (void *) &ffish);
-
         if (ffish.fish_path)
           {
             /* we have found suitable fish path in the database */
             return ffish.fish_path;
           }
+
         if (!ffish.fish_fish)
           {
             /* we haven't tried to search for suitable path yet */
@@ -231,6 +269,7 @@ babl_fish (const void *source,
               {
                 return fish_path;
               }
+#if 1
             else
               {
                 /* there isn't a suitable path for requested formats,
@@ -249,6 +288,7 @@ babl_fish (const void *source,
                 fish->fish.destination          = destination_format;
                 babl_db_insert (babl_fish_db (), fish);
               }
+#endif
           }
       }
 
