@@ -28,7 +28,7 @@
 #endif
 
 #define BABL_MAX_COMPONENTS       32
-#define BABL_CONVERSIONS           5
+#define BABL_CONVERSIONS          5
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -51,13 +51,16 @@
 #include "babl-memory.h"
 #include "babl-mutex.h"
 #include "babl-cpuaccel.h"
+#include "babl-polynomial.h"
 
 /* fallback to floor function when rint is not around */
 #ifndef HAVE_RINT
 # define rint(f)  (floor (((double) (f)) + 0.5))
 #endif
 
-
+#ifdef __ANDROID_API__
+#include <android/log.h>
+#endif
 
 Babl *   babl_conversion_find           (const void     *source,
                                          const void     *destination);
@@ -122,15 +125,31 @@ real_babl_log_va(const char *file,
   if (extender != babl_extension_quiet_log())
     {
       if (babl_extender())
-        fprintf (stdout, "When loading %s:\n\t", babl_extender()->instance.name);
+        {
+#ifdef __ANDROID_API__
+          __android_log_print (ANDROID_LOG_DEBUG, "BABL",
+                               "When loading %s:\n\t", babl_extender()->instance.name);
+#else
+          fprintf (stdout, "When loading %s:\n\t", babl_extender()->instance.name);
+#endif
+        }
 
+#ifdef __ANDROID_API__
+      __android_log_print (ANDROID_LOG_DEBUG, "BABL",
+                           "%s:%i %s()", file, line, function);
+#else
       fprintf (stdout, "%s:%i %s()\n\t", file, line, function);
+#endif
     }
 
+#ifdef __ANDROID_API__
+  __android_log_vprint (ANDROID_LOG_DEBUG, "BABL",
+                        fmt, varg);
+#else
   vfprintf (stdout, fmt, varg);
-
   fprintf (stdout, "\n");
   fflush (NULL);
+#endif
   return;
 }
 
@@ -319,10 +338,152 @@ static inline double babl_parse_double (const char *str)
   return result;
 }
 
+const Babl *
+babl_remodel_with_space (const Babl *model, const Babl *space);
+const Babl *
+babl_model_with_space (const char *name, const Babl *space);
+Babl *
+_conversion_new (const char    *name,
+                 int            id,
+                 const Babl    *source,
+                 const Babl    *destination,
+                 BablFuncLinear linear,
+                 BablFuncPlane  plane,
+                 BablFuncPlanar planar,
+                 void          *user_data);
 
 double _babl_legal_error (void);
 void babl_init_db (void);
 void babl_store_db (void);
 int _babl_max_path_len (void);
 
+
+const Babl *
+babl_trc_new (const char *name,
+              BablTRCType type,
+              double      gamma,
+              int         n_lut,
+              float      *lut);
+
+void babl_space_to_xyz   (const Babl *space, const double *rgb, double *xyz);
+void babl_space_from_xyz (const Babl *space, const double *xyz, double *rgb);
+
+const Babl *babl_trc_lut_find (float *lut, int lut_size);
+const Babl *babl_trc_lut      (const char *name, int n, float *entries);
+
+Babl * format_new_from_format_with_space (const Babl *format, const Babl *space);
+
+int babl_list_destroy (void *data);
+
+const char *
+babl_conversion_create_name (Babl *source, Babl *destination, int is_reference);
+void _babl_space_add_universal_rgb (const Babl *space);
+const Babl *
+babl_trc_formula_srgb (double gamma, double a, double b, double c, double d);
+
+
+const Babl *babl_space_match_trc_matrix (const Babl *trc_red,
+                                         const Babl *trc_green,
+                                         const Babl *trc_blue,
+                                         float rx, float ry, float rz,
+                                         float gx, float gy, float gz,
+                                         float bx, float by, float bz);
+
+/**
+ * babl_space_from_chromaticities:
+ *
+ * Creates a new babl-space/ RGB matrix color space definition with the
+ * specified CIE xy(Y) values for white point: wx, wy and primary
+ * chromaticities: rx,ry,gx,gy,bx,by and TRCs to be used. After registering a
+ * new babl-space it can be used with babl_space() passing its name;
+ *
+ * Internally this does the math to derive the RGBXYZ matrix as used in an ICC
+ * profile.
+ */
+const Babl * babl_space_from_chromaticities (const char *name,
+                                             double wx, double wy,
+                                             double rx, double ry,
+                                             double gx, double gy,
+                                             double bx, double by,
+                                             const Babl *trc_red,
+                                             const Babl *trc_green,
+                                             const Babl *trc_blue);
+
+/**
+ * babl_space_from_rgbxyz_matrix:
+ *
+ * Creates a new RGB matrix color space definition using a precomputed D50
+ * adapted 3x3 matrix and associated CIE XYZ whitepoint, as possibly read from
+ * an ICC profile.
+ */
+const Babl *
+babl_space_from_rgbxyz_matrix (const char *name,
+                               double wx, double wy, double wz,
+                               double rx, double gx, double bx,
+                               double ry, double gy, double by,
+                               double rz, double gz, double bz,
+                               const Babl *trc_red,
+                               const Babl *trc_green,
+                               const Babl *trc_blue);
+
+/**
+ * babl_trc_gamma:
+ *
+ * Creates a Babl TRC for a specific gamma value, it will be given
+ * a name that is a short string representation of the value.
+ */
+const Babl * babl_trc_gamma (double gamma);
+
+/**
+ * babl_trc:
+ *
+ * Look up a TRC by name, "sRGB" "1.0" "linear" and "2.2" are recognized
+ * strings in a stock babl configuration.
+ */
+const Babl * babl_trc       (const char *name);
+
+int _babl_file_get_contents (const char  *path,
+                             char       **contents,
+                             long        *length,
+                             void        *error);
+
+typedef enum {
+  BABL_ICC_DEFAULTS = 0,
+  BABL_ICC_COMPACT_TRC_LUT = 1,
+} BablICCFlags;
+
+/* babl_space_to_icc:
+ *
+ * Creates an ICCv2 RGB matrix profile for a babl space. The profiles strive to
+ * be as small and compact as possible, TRCs are stored as 1024 entry LUT(s).
+ *
+ * you should make a copy of the profile before making another call to this
+ * function.
+ */
+
+const char *babl_space_to_icc (const Babl  *space,
+                               const char  *description,
+                               const char  *copyright,
+                               BablICCFlags flags,
+                               int         *icc_length);
+
+/* babl_space_get_rgbtoxyz:
+
+   Returns the double-precision 3x3 matrix used to convert linear
+   RGB data to CIE XYZ.
+ */
+const double * babl_space_get_rgbtoxyz (const Babl *space);
+
+/* babl_space_to_xyz:
+ *
+ * converts a double triplet from linear RGB to CIE XYZ.
+ */
+void babl_space_to_xyz   (const Babl *space, const double *rgb, double *xyz);
+
+/* babl_space_from_xyz:
+ *
+ * converts double triplet from CIE XYZ to linear RGB
+ */
+void babl_space_from_xyz (const Babl *space, const double *xyz, double *rgb);
+
 #endif
