@@ -91,11 +91,52 @@ babl_rel_avg_error (const double *imgA,
   for (i = 0; i < samples; i++)
     error += fabs (imgA[i] - imgB[i]);
 
-  if (error >= 0.000001)
+  if (error >= 0.0000001)
     error /= samples;
-  else
+  else if (error <= 0.0)
     error = 0.0;
+  else
+    error = M_PI;
 
   return error;
 }
 
+int
+_babl_file_get_contents (const char  *path,
+                         char       **contents,
+                         long        *length,
+                         void        *error)
+{
+  FILE *file;
+  long  size;
+  char *buffer;
+
+  file = fopen (path,"rb");
+
+  if (!file)
+    return -1;
+
+  fseek (file, 0, SEEK_END);
+  size = ftell (file);
+  if (length) *length = size;
+  rewind (file);
+  buffer = calloc(size + 8, 1);
+
+  if (!buffer)
+    {
+      fclose(file);
+      return -1;
+    }
+
+  size -= fread (buffer, 1, size, file);
+  if (size)
+    {
+      fclose (file);
+      free (buffer);
+      return -1;
+    }
+  fclose (file);
+  *contents = buffer;
+  return 0;
+}
+
