@@ -31,23 +31,31 @@ extern "C" {
 
 
 /**
+ * babl_init:
+ *
  * Initializes the babl library.
  */
 void         babl_init      (void);
 
 /**
+ * babl_exit:
+ *
  * Deinitializes the babl library and frees any resources used when
  * matched with the number of calls to babl_init().
  */
 void         babl_exit      (void);
 
 /**
+ * babl_type:
+ *
  * Returns the babl object representing the data type given by @name
  * such as for example "u8", "u16" or "float".
  */
 const Babl * babl_type      (const char *name);
 
 /**
+ * babl_sampling:
+ *
  * Returns the babl object representing the @horizontal and @vertical
  * sampling such as for example 2, 2 for the chroma components in
  * YCbCr.
@@ -55,32 +63,152 @@ const Babl * babl_type      (const char *name);
 const Babl * babl_sampling  (int horizontal,
                              int vertical);
 
+
 /**
+ * babl_component:
+ *
  * Returns the babl object representing the color component given by
  * @name such as for example "R", "cyan" or "CIE L".
  */
 const Babl * babl_component (const char *name);
 
 /**
+ * babl_model:
+ *
  * Returns the babl object representing the color model given by @name
  * such as for example "RGB", "CMYK" or "CIE Lab".
  */
 const Babl * babl_model     (const char *name);
 
+
 /**
+ * babl_space:
+ *
+ * Returns the babl object representing the specific RGB matrix color
+ * working space referred to by name. Babl knows of:
+ *    sRGB, Rec2020, Adobish, Apple and ProPhoto
+ *
+ */
+const Babl * babl_space (const char *name);
+
+typedef enum {
+  BABL_ICC_INTENT_PERCEPTUAL             = 0,
+  BABL_ICC_INTENT_RELATIVE_COLORIMETRIC  = 1,
+  BABL_ICC_INTENT_SATURATION             = 2,
+  BABL_ICC_INTENT_ABSOLUTE_COLORIMETRIC  = 3
+} BablIccIntent;
+
+/**
+ * babl_icc_make_space:
+ *
+ * @icc_data: pointer to icc profile in memory
+ * @icc_length: length of icc profile in bytes
+ * @intent: the intent from the ICC profile to use.
+ *
+ " @error: pointer to a string where decoding errors can be stored,
+ *         if an error occurs, NULL is returned and an error message
+ *         is provided in error.
+ *
+ * Create a babl space from an in memory ICC profile, the profile does no
+ * longer need to be loaded for the space to work, multiple calls with the same
+ * icc profile and same intent will result in the same babl space.
+ *
+ * On a profile that doesn't contain A2B0 and B2A0 CLUTs perceptual and
+ * relative-colorimetric intents are treated the same.
+ *
+ * If a BablSpace cannot be created from the profile NULL is returned and a
+ * static string is set on the const char *value pointed at with &value
+ * containing a message describing why the provided data does not yield a babl
+ * space.
+ */
+const Babl *babl_icc_make_space (const char       *icc_data,
+                                 int               icc_length,
+                                 BablIccIntent     intent,
+                                 const char      **error);
+
+
+/* babl_icc_get_key:
+ *
+ * @icc_data: pointer to in-memory icc profile
+ * @icc_length: length of icc profile in bytes
+ * @key: the key we want to quey, see below for some supported values
+ * @language: 2 char code for language to extract or NULL
+ * @country: 2 char country code or NULL
+ *
+ * Returns NULL if key not found or a malloc allocated utf8 string of the key
+ * when found, free with free() when done. Supported keys: "description",
+ * "copyright", "manufacturer", "device", "profile-class", "color-space" and
+ * "pcs".
+ */
+
+char *babl_icc_get_key (const char *icc_data,
+                        int         icc_length,
+                        const char *key,
+                        const char *language,
+                        const char *counter);
+
+
+/**
+ * babl_format:
+ *
  * Returns the babl object representing the color format given by
  * @name such as for example "RGB u8", "CMYK float" or "CIE Lab u16".
  */
-const Babl * babl_format    (const char *name);
+const Babl * babl_format            (const char *name);
 
-/*  Create a babl fish capable of converting from source_format to
+/**
+ * babl_format_with_space:
+ *
+ * Returns the babl object representing the color format given by
+ * @name such as for example "RGB u8", "R'G'B'A float", "Y float" with
+ * a specific RGB working space used as the space, the resulting format
+ * has -space suffixed to it, unless the space requested is sRGB then
+ * the unsuffixed version is used. If a format is passed in as space
+ * the space of the format is used.
+ */
+const Babl * babl_format_with_space (const char *name, const Babl *space);
+
+/*
+ * babl_format_get_space:
+ *
+ * Retrieve the RGB color space used for a pixel format.
+ */
+const Babl * babl_format_get_space  (const Babl *format);
+
+
+/**
+ * babl_fish:
+ *
+ *  Create a babl fish capable of converting from source_format to
  *  destination_format, source and destination can be either strings
  *  with the names of the formats or Babl-format objects.
  */
 const Babl * babl_fish      (const void *source_format,
                              const void *destination_format);
 
-/** Process n pixels from source to destination using babl_fish,
+
+/**
+ * babl_fast_fish:
+ *
+ * Creae a faster than normal fish with specified performance (and thus
+ * corresponding precision tradeoff), values tolerance can hold: NULL and
+ * "default", means do same as babl_fish(), other values understood in
+ * increasing order of speed gain are:
+ *    "exact" "precise" "fast" "glitch"
+ *
+ * Fast fishes should be cached, since they are not internally kept track
+ * of/made into singletons by babl and many creations of fast fishes will
+ * otherwise be a leak.
+ *
+ */
+const Babl * babl_fast_fish (const void *source_format,
+                             const void *destination_format,
+                             const char *performance);
+
+/**
+ * babl_process:
+ *
+ *  Process n pixels from source to destination using babl_fish,
  *  returns number of pixels converted.
  */
 long         babl_process   (const Babl *babl_fish,
@@ -90,26 +218,43 @@ long         babl_process   (const Babl *babl_fish,
 
 
 /**
- * Returns a string decsribing a Babl object.
+ * babl_get_name:
+ *
+ * Returns a string describing a Babl object.
  */
 const char * babl_get_name                     (const Babl *babl);
 
 /**
+ * babl_format_has_alpha:
+ *
  * Returns whether the @format has an alpha channel.
  */
 int          babl_format_has_alpha             (const Babl *format);
 
 /**
+ * babl_format_get_bytes_per_pixel:
+ *
  * Returns the bytes per pixel for a babl color format.
  */
 int          babl_format_get_bytes_per_pixel   (const Babl *format);
 
 /**
+ * babl_format_get_model:
+ *
+ * Return the model used for constructing the format.
+ */
+const Babl * babl_format_get_model             (const Babl *format);
+
+/**
+ * babl_format_get_n_components:
+ *
  * Returns the number of components for the given @format.
  */
 int          babl_format_get_n_components      (const Babl *format);
 
 /**
+ * babl_format_get_type:
+ *
  * Returns the type in the given @format for the given
  * @component_index.
  */
@@ -118,6 +263,8 @@ const Babl * babl_format_get_type              (const Babl *format,
 
 
 /**
+ * babl_type_new:
+ *
  * Defines a new data type in babl. A data type that babl can have in
  * its buffers requires conversions to and from "double" to be
  * registered before passing sanity.
@@ -132,6 +279,8 @@ const Babl * babl_type_new (void *first_arg,
                             ...) BABL_ARG_NULL_TERMINATED;
 
 /**
+ * babl_component_new:
+ *
  * Defines a new color component with babl.
  *
  *     babl_component_new  (const char *name,
@@ -141,6 +290,8 @@ const Babl * babl_component_new  (void *first_arg,
                                   ...) BABL_ARG_NULL_TERMINATED;
 
 /**
+ * babl_model_new:
+ *
  * Defines a new color model in babl. If no name is provided a name is
  * generated by concatenating the name of all the involved components.
  *
@@ -153,6 +304,8 @@ const Babl * babl_model_new (void *first_arg,
                             ...) BABL_ARG_NULL_TERMINATED;
 
 /**
+ * babl_format_new:
+ *
  * Defines a new pixel format in babl. Provided BablType and|or
  * BablSampling is valid for the following components as well. If no
  * name is provided a (long) descriptive name is used.
@@ -173,6 +326,8 @@ const Babl * babl_format_new (const void *first_arg,
                               ...) BABL_ARG_NULL_TERMINATED;
 
 /*
+ * babl_format_n:
+ *
  * Defines a new pixel format in babl. With the specified data storage
  * type and the given number of components. At the moment behavior of
  * conversions are only well defined to other babl_format_n derived formats
@@ -183,11 +338,15 @@ babl_format_n (const Babl *type,
                int   components);
 
 /**
+ * babl_format_is_format_n:
+ *
  * Returns whether the @format is a format_n type.
  */
 int babl_format_is_format_n (const Babl *format);
 
 /**
+ * babl_conversion_new:
+ *
  * Defines a new conversion between either two formats, two models or
  * two types in babl.
  *
@@ -200,8 +359,23 @@ int babl_format_is_format_n (const Babl *format);
 const Babl * babl_conversion_new (const void *first_arg,
                                   ...) BABL_ARG_NULL_TERMINATED;
 
+/**
+ * babl_conversion_get_source_space:
+ *
+ * Returns the RGB space defined for the source of conversion.
+ */
+const Babl *babl_conversion_get_source_space      (const Babl *conversion);
 
 /**
+ * babl_conversion_get_destination_space:
+ *
+ * Returns the RGB space defined for the destination of conversion.
+ */
+const Babl *babl_conversion_get_destination_space (const Babl *conversion);
+
+/**
+ * babl_new_palette:
+ *
  * create a new palette based format, name is optional pass in NULL to get
  * an anonymous format. If you pass in with_alpha the format also gets
  * an 8bit alpha channel. Returns the BablModel of the color model. If
@@ -213,11 +387,15 @@ const Babl *babl_new_palette (const char  *name,
                               const Babl **format_u8_with_alpha);
 
 /**
+ * babl_format_is_palette:
+ *
  * check whether a format is a palette backed format.
  */
 int   babl_format_is_palette   (const Babl *format);
 
 /**
+ * babl_palette_set_palette:
+ *
  * Assign a palette to a palette format, the data is a single span of pixels
  * representing the colors of the palette.
  */
@@ -227,6 +405,8 @@ void  babl_palette_set_palette (const Babl        *babl,
                                 int                count);
 
 /**
+ * babl_palette_reset:
+ *
  * reset a palette to initial state, frees up some caches that optimize
  * conversions.
  */
@@ -235,6 +415,8 @@ void  babl_palette_reset       (const Babl        *babl);
 
 
 /**
+ * babl_set_user_data: (skip)
+ *
  * associate a data pointer with a format/model, this data can be accessed and
  * used from the conversion functions, encoding color profiles, palettes or
  * similar with the data, perhaps this should be made internal API, not
@@ -243,24 +425,25 @@ void  babl_palette_reset       (const Babl        *babl);
 void   babl_set_user_data     (const Babl *babl, void *data);
 
 /**
- * get data set with babl_set_user_data
+ * babl_get_user_data: (skip)
+ *
+ * Get data set with babl_set_user_data
  */
 void * babl_get_user_data     (const Babl *babl);
 
 
-
-/*
- * Backwards compatibility stuff
- *
- * NOTE: will most likely be removed in the first stable release!
- */
-#ifndef BABL_DISABLE_DEPRECATED
-#define babl_destroy babl_exit
-#endif
-
-
 #ifdef __cplusplus
 }
 #endif
 
 #endif
+
+/* Trademarks:
+ *
+ * International Color Consortium is a registered trademarks of the.
+ * International Color Consortium.
+ * Apple is a trademark or registered trademark of Apple Inc in many countries.
+ * Adobish is meant to concisely convey resemblence/compatibility with Adobe
+ * RGB- without actualy being it, Adobe is a trademark or registered trademark
+ * of Adobe Systems Incorporated in many countires.
+ */
