@@ -22,7 +22,7 @@
 #include <math.h>
 #include <string.h>
 
-#include "babl.h"
+#include "babl-internal.h"
 #include "extensions/util.h"
 
 #define DEGREES_PER_RADIAN (180 / 3.14159265358979323846)
@@ -137,13 +137,6 @@ static inline void  CHab_to_ab    (double  C,
                                    double  H,
                                    double *to_a,
                                    double *to_b);
-                                   
-static inline void RGB_to_XYZ     (double R,
-                                   double G,
-                                   double B,
-                                   double *to_X,
-                                   double *to_Y,
-                                   double *to_Z);
 
 static inline void XYZ_to_LAB     (double X,
                                    double Y,
@@ -161,82 +154,6 @@ static inline void LAB_to_XYZ     (double L,
                                    double *to_Z
                                    );
 
-static inline void XYZ_to_RGB     (double X,
-                                   double Y,
-                                   double Z,
-                                   double *to_R,
-                                   double *to_G,
-                                   double *to_B);
-
-static inline void
-RGB_to_XYZ (double R,
-            double G,
-            double B,
-            double *to_X,
-            double *to_Y,
-            double *to_Z)
-{
-  double RGBtoXYZ[3][3];
-
-/*
- * The variables below hard-code the D50-adapted sRGB RGB to XYZ matrix.
- *
- * In a properly ICC profile color-managed application, this matrix
- * is retrieved from the image's ICC profile's RGB colorants.
- *
- * */
-  RGBtoXYZ[0][0]= 0.43603516;
-  RGBtoXYZ[0][1]= 0.38511658;
-  RGBtoXYZ[0][2]= 0.14305115;
-  RGBtoXYZ[1][0]= 0.22248840;
-  RGBtoXYZ[1][1]= 0.71690369;
-  RGBtoXYZ[1][2]= 0.06060791;
-  RGBtoXYZ[2][0]= 0.01391602;
-  RGBtoXYZ[2][1]= 0.09706116;
-  RGBtoXYZ[2][2]= 0.71392822;
-
-/* Convert RGB to XYZ */
-  *to_X = RGBtoXYZ[0][0]*R + RGBtoXYZ[0][1]*G + RGBtoXYZ[0][2]*B;
-  *to_Y = RGBtoXYZ[1][0]*R + RGBtoXYZ[1][1]*G + RGBtoXYZ[1][2]*B;
-  *to_Z = RGBtoXYZ[2][0]*R + RGBtoXYZ[2][1]*G + RGBtoXYZ[2][2]*B;
-
-}
-
-static inline void
-XYZ_to_RGB (double X,
-            double Y,
-            double Z,
-            double *to_R,
-            double *to_G,
-            double *to_B)
-{
-  double XYZtoRGB[3][3];
-
-/*
- * The variables below hard-code the inverse of
- * the D50-adapted sRGB RGB to XYZ matrix.
- *
- * In a properly ICC profile color-managed application,
- * this matrix is the inverse of the matrix
- * retrieved from the image's ICC profile's RGB colorants.
- *
- */
-  XYZtoRGB[0][0]=  3.134274799724;
-  XYZtoRGB[0][1]= -1.617275708956;
-  XYZtoRGB[0][2]= -0.490724283042;
-  XYZtoRGB[1][0]= -0.978795575994;
-  XYZtoRGB[1][1]=  1.916161689117;
-  XYZtoRGB[1][2]=  0.033453331711;
-  XYZtoRGB[2][0]=  0.071976988401;
-  XYZtoRGB[2][1]= -0.228984974402;
-  XYZtoRGB[2][2]=  1.405718224383;
-
-/* Convert XYZ to RGB */
-  *to_R = XYZtoRGB[0][0] * X + XYZtoRGB[0][1] * Y + XYZtoRGB[0][2] * Z;
-  *to_G = XYZtoRGB[1][0] * X + XYZtoRGB[1][1] * Y + XYZtoRGB[1][2] * Z;
-  *to_B = XYZtoRGB[2][0] * X + XYZtoRGB[2][1] * Y + XYZtoRGB[2][2] * Z;
-}
-
 static inline void
 XYZ_to_LAB (double X,
             double Y,
@@ -299,23 +216,24 @@ LAB_to_XYZ (double L,
   *to_Z = zr * D50_WHITE_REF_Z;
 }
 
-static long
-rgba_to_lab (char *src,
+static void
+rgba_to_lab (const Babl *conversion,char *src,
              char *dst,
              long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
   while (n--)
     {
-      double R  = ((double *) src)[0];
-      double G  = ((double *) src)[1];
-      double B  = ((double *) src)[2];
-      double X, Y, Z, L, a, b;
-      
+      double RGB[3]  = {((double *) src)[0],
+                        ((double *) src)[1],
+                        ((double *) src)[2] };
+      double XYZ[3], L, a, b;
+
       //convert RGB to XYZ
-      RGB_to_XYZ (R, G, B, &X, &Y, &Z);
-      
+      babl_space_to_xyz (space, RGB, XYZ);
+
       //convert XYZ to Lab
-      XYZ_to_LAB (X, Y, Z, &L, &a, &b);
+      XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
 
       ((double *) dst)[0] = L;
       ((double *) dst)[1] = a;
@@ -324,14 +242,14 @@ rgba_to_lab (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 3;
     }
-  return n;
 }
 
-static long
-lab_to_rgba (char *src,
+static void
+lab_to_rgba (const Babl *conversion,char *src,
              char *dst,
              long  n)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
   while (n--)
     {
       double L = ((double *) src)[0];
@@ -344,7 +262,15 @@ lab_to_rgba (char *src,
       LAB_to_XYZ (L, a, b, &X, &Y, &Z);
 
       //convert XYZ to RGB
-      XYZ_to_RGB (X, Y, Z, &R, &G, &B);
+      {
+        double XYZ[3]  = {X,Y,Z};
+        double RGB[3];
+        babl_space_from_xyz (space, XYZ, RGB);
+        R = RGB[0];
+        G = RGB[1];
+        B = RGB[2];
+      }
+
       ((double *) dst)[0] = R;
       ((double *) dst)[1] = G;
       ((double *) dst)[2] = B;
@@ -353,27 +279,27 @@ lab_to_rgba (char *src,
       src += sizeof (double) * 3;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
-static long
-rgba_to_laba (char *src,
+static void
+rgba_to_laba (const Babl *conversion,char *src,
               char *dst,
               long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
   while (n--)
     {
-      double R     = ((double *) src)[0];
-      double G     = ((double *) src)[1];
-      double B     = ((double *) src)[2];
+      double RGB[3]  = {((double *) src)[0],
+                        ((double *) src)[1],
+                        ((double *) src)[2] };
       double alpha = ((double *) src)[3];
-      double X, Y, Z, L, a, b;
+      double XYZ[3], L, a, b;
 
       //convert RGB to XYZ
-      RGB_to_XYZ (R, G, B, &X, &Y, &Z);
+      babl_space_to_xyz (space, RGB, XYZ);
 
       //convert XYZ to Lab
-      XYZ_to_LAB (X, Y, Z, &L, &a, &b);
+      XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
 
       ((double *) dst)[0] = L;
       ((double *) dst)[1] = a;
@@ -383,14 +309,14 @@ rgba_to_laba (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
-static long
-laba_to_rgba (char *src,
+static void
+laba_to_rgba (const Babl *conversion,char *src,
               char *dst,
               long  n)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
   while (n--)
     {
       double L     = ((double *) src)[0];
@@ -404,7 +330,17 @@ laba_to_rgba (char *src,
       LAB_to_XYZ (L, a, b, &X, &Y, &Z);
 
       //convert XYZ to RGB
-      XYZ_to_RGB (X, Y, Z, &R, &G, &B);
+      //XYZ_to_RGB (X, Y, Z, &R, &G, &B);
+
+      {
+        double XYZ[3]  = {X,Y,Z};
+        double RGB[3];
+        babl_space_from_xyz (space, XYZ, RGB);
+        R = RGB[0];
+        G = RGB[1];
+        B = RGB[2];
+      }
+
       ((double *) dst)[0] = R;
       ((double *) dst)[1] = G;
       ((double *) dst)[2] = B;
@@ -413,7 +349,6 @@ laba_to_rgba (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
 static inline void
@@ -440,23 +375,26 @@ ab_to_CHab (double  a,
       *to_H += 360;
 }
 
-static long
-rgba_to_lchab (char *src,
+static void
+rgba_to_lchab (const Babl *conversion,char *src,
                char *dst,
                long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+
   while (n--)
     {
-      double R = ((double *) src)[0];
-      double G = ((double *) src)[1];
-      double B = ((double *) src)[2];
-      double X, Y, Z, L, a, b, C, H;
+      double RGB[3]  = {((double *) src)[0],
+                        ((double *) src)[1],
+                        ((double *) src)[2] };
+      double XYZ[3], L, a, b, C, H;
 
       //convert RGB to XYZ
-      RGB_to_XYZ (R, G, B, &X, &Y, &Z);
+      babl_space_to_xyz (space, RGB, XYZ);
 
       //convert XYZ to Lab
-      XYZ_to_LAB (X, Y, Z, &L, &a, &b);
+      XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
+
 
       //convert Lab to LCH(ab)
       ab_to_CHab (a, b, &C, &H);
@@ -468,14 +406,15 @@ rgba_to_lchab (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 3;
     }
-  return n;
 }
 
-static long
-lchab_to_rgba (char *src,
+static void
+lchab_to_rgba (const Babl *conversion,char *src,
                char *dst,
                long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+
   while (n--)
     {
       double L = ((double *) src)[0];
@@ -490,7 +429,15 @@ lchab_to_rgba (char *src,
       LAB_to_XYZ (L, a, b, &X, &Y, &Z);
 
       //Convert XYZ to RGB
-      XYZ_to_RGB (X, Y, Z, &R, &G, &B);
+      {
+        double XYZ[3]  = {X,Y,Z};
+        double RGB[3];
+        babl_space_from_xyz (space, XYZ, RGB);
+        R = RGB[0];
+        G = RGB[1];
+        B = RGB[2];
+      }
+
 
       ((double *) dst)[0] = R;
       ((double *) dst)[1] = G;
@@ -500,27 +447,28 @@ lchab_to_rgba (char *src,
       src += sizeof (double) * 3;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
-static long
-rgba_to_lchaba (char *src,
+static void
+rgba_to_lchaba (const Babl *conversion,char *src,
                 char *dst,
                 long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+
   while (n--)
     {
-      double R = ((double *) src)[0];
-      double G = ((double *) src)[1];
-      double B = ((double *) src)[2];
+      double RGB[3]  = {((double *) src)[0],
+                        ((double *) src)[1],
+                        ((double *) src)[2] };
       double alpha = ((double *) src)[3];
-      double X, Y, Z, L, a, b, C, H;
+      double XYZ[3], L, a, b, C, H;
 
       //convert RGB to XYZ
-      RGB_to_XYZ (R, G, B, &X, &Y, &Z);
+      babl_space_to_xyz (space, RGB, XYZ);
 
       //convert XYZ to Lab
-      XYZ_to_LAB (X, Y, Z, &L, &a, &b);
+      XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
 
       //convert Lab to LCH(ab)
       ab_to_CHab (a, b, &C, &H);
@@ -533,14 +481,14 @@ rgba_to_lchaba (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
-static long
-lchaba_to_rgba (char *src,
+static void
+lchaba_to_rgba (const Babl *conversion,char *src,
                 char *dst,
                 long  n)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
   while (n--)
     {
       double L     = ((double *) src)[0];
@@ -556,7 +504,14 @@ lchaba_to_rgba (char *src,
       LAB_to_XYZ (L, a, b, &X, &Y, &Z);
 
       //Convert XYZ to RGB
-      XYZ_to_RGB (X, Y, Z, &R, &G, &B);
+      {
+        double XYZ[3]  = {X,Y,Z};
+        double RGB[3];
+        babl_space_from_xyz (space, XYZ, RGB);
+        R = RGB[0];
+        G = RGB[1];
+        B = RGB[2];
+      }
 
       ((double *) dst)[0] = R;
       ((double *) dst)[1] = G;
@@ -566,7 +521,6 @@ lchaba_to_rgba (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
 
@@ -637,8 +591,8 @@ cubef (float f)
   return f * f * f;
 }
 
-static long
-Yaf_to_Laf (float *src,
+static void
+Yaf_to_Laf (const Babl *conversion,float *src,
             float *dst,
             long   samples)
 {
@@ -656,12 +610,10 @@ Yaf_to_Laf (float *src,
       src += 2;
       dst += 2;
     }
-
-  return samples;
 }
 
-static long
-rgbf_to_Labf (float *src,
+static void
+rgbf_to_Labf (const Babl *conversion,float *src,
               float *dst,
               long   samples)
 {
@@ -692,12 +644,10 @@ rgbf_to_Labf (float *src,
       src += 3;
       dst += 3;
     }
-
-  return samples;
 }
 
-static long
-rgbaf_to_Labaf (float *src,
+static void
+rgbaf_to_Labaf (const Babl *conversion,float *src,
                 float *dst,
                 long   samples)
 {
@@ -730,12 +680,10 @@ rgbaf_to_Labaf (float *src,
       src += 4;
       dst += 4;
     }
-
-  return samples;
 }
 
-static long
-Labf_to_rgbf (float *src,
+static void
+Labf_to_rgbf (const Babl *conversion,float *src,
                 float *dst,
                 long   samples)
 {
@@ -766,12 +714,10 @@ Labf_to_rgbf (float *src,
       src += 3;
       dst += 3;
     }
-
-  return samples;
 }
 
-static long
-Labaf_to_rgbaf (float *src,
+static void
+Labaf_to_rgbaf (const Babl *conversion,float *src,
                 float *dst,
                 long   samples)
 {
@@ -804,8 +750,118 @@ Labaf_to_rgbaf (float *src,
       src += 4;
       dst += 4;
     }
+}
 
-  return samples;
+static void
+Labf_to_Lchabf (const Babl *conversion,float *src,
+                float *dst,
+                long   samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      float L = src[0];
+      float A = src[1];
+      float B = src[2];
+
+      float C = sqrtf (A * A + B * B);
+      float H = atan2f (B, A) * DEGREES_PER_RADIAN;
+
+      // Keep H within the range 0-360
+      if (H < 0.0f)
+        H += 360.0f;
+
+      dst[0] = L;
+      dst[1] = C;
+      dst[2] = H;
+
+      src += 3;
+      dst += 3;
+    }
+}
+
+static void
+Lchabf_to_Labf (const Babl *conversion,float *src,
+                float *dst,
+                long   samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      float L = src[0];
+      float C = src[1];
+      float H = src[2];
+
+      float A = C * cosf (H * RADIANS_PER_DEGREE);
+      float B = C * sinf (H * RADIANS_PER_DEGREE);
+
+      dst[0] = L;
+      dst[1] = A;
+      dst[2] = B;
+
+      src += 3;
+      dst += 3;
+    }
+}
+
+static void
+Labaf_to_Lchabaf (const Babl *conversion,float *src,
+                  float *dst,
+                  long   samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      float L = src[0];
+      float A = src[1];
+      float B = src[2];
+      float a = src[3];
+
+      float C = sqrtf (A * A + B * B);
+      float H = atan2f (B, A) * DEGREES_PER_RADIAN;
+
+      // Keep H within the range 0-360
+      if (H < 0.0f)
+        H += 360.0f;
+
+      dst[0] = L;
+      dst[1] = C;
+      dst[2] = H;
+      dst[3] = a;
+
+      src += 4;
+      dst += 4;
+    }
+}
+
+static void
+Lchabaf_to_Labaf (const Babl *conversion,float *src,
+                  float *dst,
+                  long   samples)
+{
+  long n = samples;
+
+  while (n--)
+    {
+      float L = src[0];
+      float C = src[1];
+      float H = src[2];
+      float a = src[3];
+
+      float A = C * cosf (H * RADIANS_PER_DEGREE);
+      float B = C * sinf (H * RADIANS_PER_DEGREE);
+
+      dst[0] = L;
+      dst[1] = A;
+      dst[2] = B;
+      dst[3] = a;
+
+      src += 4;
+      dst += 4;
+    }
 }
 
 static void
@@ -889,6 +945,30 @@ conversions (void)
     "linear", lchaba_to_rgba,
     NULL
   );
+  babl_conversion_new (
+    babl_format ("CIE Lab float"),
+    babl_format ("CIE LCH(ab) float"),
+    "linear", Labf_to_Lchabf,
+    NULL
+  );
+  babl_conversion_new (
+    babl_format ("CIE LCH(ab) float"),
+    babl_format ("CIE Lab float"),
+    "linear", Lchabf_to_Labf,
+    NULL
+  );
+  babl_conversion_new (
+    babl_format ("CIE Lab alpha float"),
+    babl_format ("CIE LCH(ab) alpha float"),
+    "linear", Labaf_to_Lchabaf,
+    NULL
+  );
+  babl_conversion_new (
+    babl_format ("CIE LCH(ab) alpha float"),
+    babl_format ("CIE Lab alpha float"),
+    "linear", Lchabaf_to_Labaf,
+    NULL
+  );
   /*babl_conversion_new (
     babl_model ("RGBA"),
     babl_model ("CIE XYZ"),
@@ -998,9 +1078,10 @@ formats (void)
 
 /******** begin  integer RGB/CIE color space conversions **************/
 
-static inline long
-convert_double_u8_scaled (double        min_val,
-                          double        max_val,
+static inline void
+convert_double_u8_scaled (const Babl *conversion,
+                          double          min_val,
+                          double          max_val,
                           unsigned char min,
                           unsigned char max,
                           char         *src,
@@ -1025,11 +1106,11 @@ convert_double_u8_scaled (double        min_val,
       src                   += src_pitch;
       dst                   += dst_pitch;
     }
-  return n;
 }
 
-static inline long
-convert_u8_double_scaled (double        min_val,
+static inline void
+convert_u8_double_scaled (const Babl *conversion,
+                          double        min_val,
                           double        max_val,
                           unsigned char min,
                           unsigned char max,
@@ -1056,29 +1137,28 @@ convert_u8_double_scaled (double        min_val,
       dst += dst_pitch;
       src += src_pitch;
     }
-  return n;
 }
 
 #define MAKE_CONVERSIONS(name, min_val, max_val, min, max) \
-  static long \
-  convert_ ## name ## _double (char *src, \
+  static void \
+  convert_ ## name ## _double (const Babl *c, char *src, \
                                char *dst, \
                                int src_pitch, \
                                int dst_pitch, \
                                long n)        \
   { \
-    return convert_u8_double_scaled (min_val, max_val, min, max, \
-                                     src, dst, src_pitch, dst_pitch, n); \
+    convert_u8_double_scaled (c, min_val, max_val, min, max, \
+                              src, dst, src_pitch, dst_pitch, n); \
   }                                                               \
-  static long \
-  convert_double_ ## name (char *src, \
+  static void \
+  convert_double_ ## name (const Babl *c, char *src, \
                            char *dst, \
                            int src_pitch, \
                            int dst_pitch, \
                            long n)        \
   { \
-    return convert_double_u8_scaled (min_val, max_val, min, max, \
-                                     src, dst, src_pitch, dst_pitch, n); \
+    convert_double_u8_scaled (c, min_val, max_val, min, max, \
+                              src, dst, src_pitch, dst_pitch, n); \
   }
 
 /* source ICC.1:2004-10 */
@@ -1138,8 +1218,9 @@ types_u8 (void)
   );
 }
 
-static inline long
-convert_double_u16_scaled (double         min_val,
+static inline void
+convert_double_u16_scaled (const Babl *conversion,
+                           double         min_val,
                            double         max_val,
                            unsigned short min,
                            unsigned short max,
@@ -1165,11 +1246,11 @@ convert_double_u16_scaled (double         min_val,
       dst                    += dst_pitch;
       src                    += src_pitch;
     }
-  return n;
 }
 
-static inline long
-convert_u16_double_scaled (double         min_val,
+static inline void
+convert_u16_double_scaled (const Babl *conversion,
+                           double         min_val,
                            double         max_val,
                            unsigned short min,
                            unsigned short max,
@@ -1195,29 +1276,28 @@ convert_u16_double_scaled (double         min_val,
       dst              += dst_pitch;
       src              += src_pitch;
     }
-  return n;
 }
 
 #define MAKE_CONVERSIONS(name, min_val, max_val, min, max)      \
-  static long \
-  convert_ ## name ## _double (char *src, \
+  static void \
+  convert_ ## name ## _double (const Babl *c, char *src, \
                                char *dst, \
                                int src_pitch, \
                                int dst_pitch, \
                                long n)        \
   { \
-    return convert_u16_double_scaled (min_val, max_val, min, max, \
-                                      src, dst, src_pitch, dst_pitch, n); \
+    convert_u16_double_scaled (c, min_val, max_val, min, max, \
+                               src, dst, src_pitch, dst_pitch, n); \
   }                                                               \
-  static long \
-  convert_double_ ## name (char *src, \
+  static void \
+  convert_double_ ## name (const Babl *c, char *src, \
                            char *dst, \
                            int src_pitch, \
                            int dst_pitch, \
                            long n)        \
   { \
-    return convert_double_u16_scaled (min_val, max_val, min, max, \
-                                      src, dst, src_pitch, dst_pitch, n); \
+    convert_double_u16_scaled (c, min_val, max_val, min, max, \
+                               src, dst, src_pitch, dst_pitch, n); \
   }
 
 MAKE_CONVERSIONS (u16_l, 0.0, 100.0, 0x00, 0xffff)
