@@ -19,14 +19,15 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
-#include "babl.h"
+#include "config.h"
+#include "babl-internal.h"
 
 #include "base/util.h"
 
 int init (void);
 
-static inline long
-conv_rgba8_cairo24_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_rgba8_cairo24_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   uint32_t *srci = (void *)src;
@@ -41,11 +42,10 @@ conv_rgba8_cairo24_le (unsigned char *src, unsigned char *dst, long samples)
       uint32_t blue        = red_blue >> 16;
       *dsti++              = green_alpha | red | blue | 0xff000000;
     }
-  return samples;
 }
 
-static inline long
-conv_rgb8_cairo24_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_rgb8_cairo24_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -58,14 +58,13 @@ conv_rgb8_cairo24_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = red;
       *dst++ = 255;
     }
-  return samples;
 
 
 }
 
 #if 0
-static inline long
-conv_rgbA8_premul_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_rgbA8_premul_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -80,12 +79,11 @@ conv_rgbA8_premul_cairo32_le (unsigned char *src, unsigned char *dst, long sampl
       *dst++ = red;
       *dst++ = alpha;
     }
-  return samples;
 }
 #else
 
-static inline long
-conv_rgbA8_premul_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_rgbA8_premul_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   uint32_t *srci = (void *)src;
@@ -100,11 +98,10 @@ conv_rgbA8_premul_cairo32_le (unsigned char *src, unsigned char *dst, long sampl
       uint32_t blue        = red_blue >> 16;
       *dsti++              = green_alpha | red | blue;
     }
-  return samples;
 }
 #endif
 
-static long conv_cairo32_rgbA8_premul_le (unsigned char *src, unsigned char *dst, long samples)
+static void conv_cairo32_rgbA8_premul_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -119,11 +116,10 @@ static long conv_cairo32_rgbA8_premul_le (unsigned char *src, unsigned char *dst
       *dst++ = blue;
       *dst++ = alpha;
     }
-  return samples;
 }
 
 
-static long conv_cairo32_rgbAF_premul_le (unsigned char *src, unsigned char *dst_char, long samples)
+static void conv_cairo32_rgbAF_premul_le (const Babl *conversion,unsigned char *src, unsigned char *dst_char, long samples)
 {
   long n = samples;
   float *dst = (void*)dst_char;
@@ -139,11 +135,10 @@ static long conv_cairo32_rgbAF_premul_le (unsigned char *src, unsigned char *dst
       *dst++ = blue / 255.0;
       *dst++ = alpha / 255.0;
     }
-  return samples;
 }
 
-static inline long
-conv_rgbA8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_rgbA8_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -159,11 +154,10 @@ conv_rgbA8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = div_255 (red   * alpha);
       *dst++ = alpha;
     }
-  return samples;
 }
 
-static inline long
-conv_rgb8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_rgb8_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -177,14 +171,13 @@ conv_rgb8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = red;
       *dst++ = 0xff;
     }
-  return samples;
 }
 
 
 
 
-static inline long
-conv_yA8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_yA8_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -200,11 +193,10 @@ conv_yA8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = val;
       *dst++ = alpha;
     }
-  return samples;
 }
 
-static inline long
-conv_yA16_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_yA16_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   uint16_t *ssrc = (void*) src;
@@ -218,11 +210,10 @@ conv_yA16_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = (alpha * 0xff + 0.5f);
       ssrc+=2;
     }
-  return samples;
 }
 
-static inline long
-conv_y8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_y8_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   while (n--)
@@ -233,11 +224,10 @@ conv_y8_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = val;
       *dst++ = 0xff;
     }
-  return samples;
 }
 
-static inline long
-conv_y16_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
+static inline void
+conv_y16_cairo32_le (const Babl *conversion,unsigned char *src, unsigned char *dst, long samples)
 {
   long n = samples;
   uint16_t *s16 = (void*)src;
@@ -252,11 +242,10 @@ conv_y16_cairo32_le (unsigned char *src, unsigned char *dst, long samples)
       *dst++ = val;
       *dst++ = 0xff;
     }
-  return samples;
 }
 
-static long
-conv_rgbA_gamma_float_cairo32_le (unsigned char *src,
+static void
+conv_rgbA_gamma_float_cairo32_le (const Babl *conversion,unsigned char *src,
                                   unsigned char *dst,
                                   long           samples)
 {
@@ -276,14 +265,16 @@ conv_rgbA_gamma_float_cairo32_le (unsigned char *src,
       *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
       fsrc+=4;
     }
-  return samples;
 }
 
-static long
-conv_rgbafloat_cairo32_le (unsigned char *src,
+static void
+conv_rgbafloat_cairo32_le (const Babl *conversion,unsigned char *src,
                            unsigned char *dst,
                            long           samples)
 {
+  const Babl  *space = babl_conversion_get_destination_space (conversion);
+  const Babl **trc   = (void*)space->space.trc;
+
   float *fsrc = (float *) src;
   unsigned char *cdst = (unsigned char *) dst;
   int n = samples;
@@ -296,11 +287,11 @@ conv_rgbafloat_cairo32_le (unsigned char *src,
       float alpha  = *fsrc++;
       if (alpha >= 1.0)
       {
-        int val = babl_linear_to_gamma_2_2f (blue) * 0xff + 0.5f;
+        int val = babl_trc_from_linear (trc[2], blue) * 0xff + 0.5f;
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
-        val = babl_linear_to_gamma_2_2f (green) * 0xff + 0.5f;
+        val = babl_trc_from_linear (trc[1], green) * 0xff + 0.5f;
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
-        val = babl_linear_to_gamma_2_2f (red) * 0xff + 0.5f;
+        val = babl_trc_from_linear (trc[0], red) * 0xff + 0.5f;
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
         *cdst++ = 0xff;
       }
@@ -312,24 +303,25 @@ conv_rgbafloat_cairo32_le (unsigned char *src,
       else
       {
         float balpha = alpha * 0xff;
-        int val = babl_linear_to_gamma_2_2f (blue) * balpha + 0.5f;
+        int val = babl_trc_from_linear (trc[2], blue) * balpha + 0.5f;
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
-        val = babl_linear_to_gamma_2_2f (green) * balpha + 0.5f;
+        val = babl_trc_from_linear (trc[1], green) * balpha + 0.5f;
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
-        val = babl_linear_to_gamma_2_2f (red) * balpha + 0.5f;
+        val = babl_trc_from_linear (trc[0], red) * balpha + 0.5f;
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
         *cdst++ = balpha + 0.5f;
       }
     }
-  return samples;
 }
 
 
-static long
-conv_yafloat_cairo32_le (unsigned char *src,
+static void
+conv_yafloat_cairo32_le (const Babl *conversion,unsigned char *src,
                          unsigned char *dst,
                          long           samples)
 {
+  const Babl  *space = babl_conversion_get_destination_space (conversion);
+  const Babl **trc   = (void*)space->space.trc;
   float *fsrc = (float *) src;
   unsigned char *cdst = (unsigned char *) dst;
   int n = samples;
@@ -340,7 +332,7 @@ conv_yafloat_cairo32_le (unsigned char *src,
       float alpha  = *fsrc++;
       if (alpha >= 1.0)
       {
-        int val = babl_linear_to_gamma_2_2f (gray) * 0xff + 0.5f;
+        int val = babl_trc_from_linear (trc[0], gray) * 0xff + 0.5f;
         val = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
         *cdst++ = val;
         *cdst++ = val;
@@ -355,7 +347,7 @@ conv_yafloat_cairo32_le (unsigned char *src,
       else
       {
         float balpha = alpha * 0xff;
-        int val = babl_linear_to_gamma_2_2f (gray) * balpha + 0.5f;
+        int val = babl_trc_from_linear (trc[0], gray) * balpha + 0.5f;
         val = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
         *cdst++ = val;
         *cdst++ = val;
@@ -363,7 +355,6 @@ conv_yafloat_cairo32_le (unsigned char *src,
         *cdst++ = balpha + 0.5f;
       }
     }
-  return samples;
 }
 
 int
@@ -418,7 +409,6 @@ init (void)
       babl_conversion_new (babl_format ("Y'A u16"), f32, "linear",
                            conv_yA16_cairo32_le, NULL);
 
-
       babl_conversion_new (babl_format ("Y' u8"), f32, "linear",
                            conv_y8_cairo32_le, NULL);
       babl_conversion_new (babl_format ("Y' u16"), f32, "linear",
