@@ -287,8 +287,8 @@ babl_lookup_free (BablLookup *lookup)
 }
 #endif
 
-static INLINE long
-conv_rgbaF_linear_rgbAF_gamma (unsigned char *src, 
+static INLINE void
+conv_rgbaF_linear_rgbAF_gamma (const Babl *conversion,unsigned char *src, 
                                unsigned char *dst, 
                                long           samples)
 {
@@ -324,13 +324,12 @@ conv_rgbaF_linear_rgbAF_gamma (unsigned char *src,
          *fdst++ = alpha;
        }
      }
-  return samples;
 }
 
 
 
-static INLINE long
-conv_rgbaF_linear_rgba8_gamma (unsigned char *src, 
+static INLINE void
+conv_rgbaF_linear_rgba8_gamma (const Babl *conversion,unsigned char *src, 
                                unsigned char *dst, 
                                long           samples)
 {
@@ -363,11 +362,10 @@ conv_rgbaF_linear_rgba8_gamma (unsigned char *src,
        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
        }
      }
-  return samples;
 }
 
-static INLINE long
-conv_rgbaF_linear_rgbA8_gamma (unsigned char *src, 
+static INLINE void
+conv_rgbaF_linear_rgbA8_gamma (const Babl *conversion,unsigned char *src, 
                                unsigned char *dst, 
                                long           samples)
 {
@@ -408,11 +406,10 @@ conv_rgbaF_linear_rgbA8_gamma (unsigned char *src,
          *cdst++ = balpha + 0.5f;
        }
      }
-  return samples;
 }
 
-static INLINE long
-conv_yaF_linear_rgbA8_gamma (unsigned char *src, 
+static INLINE void
+conv_yaF_linear_rgbA8_gamma (const Babl *conversion,unsigned char *src, 
                              unsigned char *dst, 
                              long           samples)
 {
@@ -447,13 +444,12 @@ conv_yaF_linear_rgbA8_gamma (unsigned char *src,
          *cdst++ = balpha + 0.5f;
        }
      }
-  return samples;
 }
 
 
 
-static long
-conv_rgbaF_linear_rgbA8_gamma_cairo (unsigned char *src, 
+static void
+conv_rgbaF_linear_rgbA8_gamma_cairo (const Babl *conversion,unsigned char *src, 
                                      unsigned char *dst, 
                                      long           samples)
 {
@@ -494,11 +490,10 @@ conv_rgbaF_linear_rgbA8_gamma_cairo (unsigned char *src,
         *cdst++ = balpha + 0.5f;
       }
     }
-  return samples;
 }
 
-static INLINE long
-conv_rgbAF_linear_rgbAF_gamma (unsigned char *src, 
+static INLINE void
+conv_rgbAF_linear_rgbAF_gamma (const Babl *conversion,unsigned char *src, 
                                unsigned char *dst, 
                                long           samples)
 {
@@ -535,11 +530,10 @@ conv_rgbAF_linear_rgbAF_gamma (unsigned char *src,
            *fdst++ = alpha;
          }
      }
-  return samples;
 }
 
-static INLINE long
-conv_rgbaF_linear_rgbaF_gamma (unsigned char *src, 
+static INLINE void
+conv_rgbaF_linear_rgbaF_gamma (const Babl *conversion,unsigned char *src, 
                                unsigned char *dst, 
                                long           samples)
 {
@@ -554,11 +548,10 @@ conv_rgbaF_linear_rgbaF_gamma (unsigned char *src,
        *fdst++ = linear_to_gamma_2_2_lut (*fsrc++);
        *fdst++ = *fsrc++;
      }
-  return samples;
 }
 
-static INLINE long
-conv_rgbF_linear_rgbF_gamma (unsigned char *src, 
+static INLINE void
+conv_rgbF_linear_rgbF_gamma (const Babl *conversion,unsigned char *src, 
                              unsigned char *dst, 
                              long           samples)
 {
@@ -572,12 +565,11 @@ conv_rgbF_linear_rgbF_gamma (unsigned char *src,
        *fdst++ = linear_to_gamma_2_2_lut (*fsrc++);
        *fdst++ = linear_to_gamma_2_2_lut (*fsrc++);
      }
-  return samples;
 }
 
 
-static INLINE long
-conv_rgbaF_gamma_rgbaF_linear (unsigned char *src, 
+static INLINE void
+conv_rgbaF_gamma_rgbaF_linear (const Babl *conversion,unsigned char *src, 
                                unsigned char *dst, 
                                long           samples)
 {
@@ -592,11 +584,10 @@ conv_rgbaF_gamma_rgbaF_linear (unsigned char *src,
        *fdst++ = gamma_2_2_to_linear_lut (*fsrc++);
        *fdst++ = *fsrc++;
      }
-  return samples;
 }
 
-static INLINE long
-conv_rgbF_gamma_rgbF_linear (unsigned char *src, 
+static INLINE void
+conv_rgbF_gamma_rgbF_linear (const Babl *conversion,unsigned char *src, 
                              unsigned char *dst, 
                              long           samples)
 {
@@ -610,7 +601,6 @@ conv_rgbF_gamma_rgbF_linear (unsigned char *src,
        *fdst++ = gamma_2_2_to_linear_lut (*fsrc++);
        *fdst++ = gamma_2_2_to_linear_lut (*fsrc++);
      }
-  return samples;
 }
 
 #define o(src, dst) \
