@@ -37,8 +37,8 @@
 
 static const float BABL_ALPHA_THRESHOLD_FLOAT = (float)BABL_ALPHA_THRESHOLD;
 
-static long
-conv_rgbaF_linear_rgbAF_linear (const float *src, float *dst, long samples)
+static void
+conv_rgbaF_linear_rgbAF_linear (const Babl *conversion,const float *src, float *dst, long samples)
 {
   long i = 0;
   long remainder;
@@ -91,12 +91,10 @@ conv_rgbaF_linear_rgbAF_linear (const float *src, float *dst, long samples)
     src += 4;
     dst += 4;
   }
-
-  return samples;
 }
 
-static long
-conv_rgbAF_linear_rgbaF_linear_shuffle (const float *src, float *dst, long samples)
+static void
+conv_rgbAF_linear_rgbaF_linear_shuffle (const Babl *conversion,const float *src, float *dst, long samples)
 {
   long i = 0;
   long remainder;
@@ -159,12 +157,10 @@ conv_rgbAF_linear_rgbaF_linear_shuffle (const float *src, float *dst, long sampl
       src   += 4;
       dst   += 4;
     }
-
-  return samples;
 }
 
-static long
-conv_rgbAF_linear_rgbaF_linear_spin (const float *src, float *dst, long samples)
+static void
+conv_rgbAF_linear_rgbaF_linear_spin (const Babl *conversion,const float *src, float *dst, long samples)
 {
   long i = 0;
   long remainder;
@@ -234,8 +230,6 @@ conv_rgbAF_linear_rgbaF_linear_spin (const float *src, float *dst, long samples)
       src   += 4;
       dst   += 4;
     }
-
-  return samples;
 }
 
 #define splat4f(x) ((__v4sf){x,x,x,x})
@@ -296,8 +290,8 @@ gamma_2_2_to_linear_sse2 (__v4sf x)
 }
 
 #define GAMMA_RGBA(func, munge) \
-static inline long \
-func (const float *src, float *dst, long samples)\
+static inline void \
+func (const Babl *conversion,const float *src, float *dst, long samples)\
 {\
   int i = samples;\
   if (((uintptr_t)src % 16) + ((uintptr_t)dst % 16) == 0)\
@@ -347,18 +341,16 @@ func (const float *src, float *dst, long samples)\
           dst[3] = a;\
         }\
     }\
-  return samples;\
 }
 
 GAMMA_RGBA(conv_rgbaF_linear_rgbaF_gamma, linear_to_gamma_2_2_sse2)
 GAMMA_RGBA(conv_rgbaF_gamma_rgbaF_linear, gamma_2_2_to_linear_sse2)
 
-static long conv_rgbaF_linear_rgbAF_gamma (const float *src, float *dst, long samples)
+static void conv_rgbaF_linear_rgbAF_gamma (const Babl *conversion,const float *src, float *dst, long samples)
 {
   float *tmp = alloca (sizeof(float)*4*samples);
-  conv_rgbaF_linear_rgbaF_gamma (src, tmp, samples);
-  conv_rgbaF_linear_rgbAF_linear (tmp, dst, samples);
-  return samples;
+  conv_rgbaF_linear_rgbaF_gamma (conversion, src, tmp, samples);
+  conv_rgbaF_linear_rgbAF_linear (conversion, tmp, dst, samples);
 }
 
 #define YA_APPLY(load, store, convert) \
@@ -379,11 +371,9 @@ static long conv_rgbaF_linear_rgbAF_gamma (const float *src, float *dst, long sa
   store ((float *)d++, yaya1); \
 }\
 
-static long
-conv_yaF_linear_yaF_gamma (const float *src, float *dst, long samples)
+static void
+conv_yaF_linear_yaF_gamma (const Babl *conversion,const float *src, float *dst, long samples)
 {
-  long total = samples;
-
   const __v4sf *s = (const __v4sf*)src;
         __v4sf *d = (__v4sf*)dst;
 
@@ -412,16 +402,12 @@ conv_yaF_linear_yaF_gamma (const float *src, float *dst, long samples)
       *dst++ = babl_linear_to_gamma_2_2 (*src++);
       *dst++ = *src++;
     }
-
-  return total;
 }
 
 
-static long
-conv_yaF_gamma_yaF_linear (const float *src, float *dst, long samples)
+static void
+conv_yaF_gamma_yaF_linear (const Babl *conversion,const float *src, float *dst, long samples)
 {
-  long total = samples;
-
   const __v4sf *s = (const __v4sf*)src;
         __v4sf *d = (__v4sf*)dst;
 
@@ -450,15 +436,11 @@ conv_yaF_gamma_yaF_linear (const float *src, float *dst, long samples)
       *dst++ = babl_gamma_2_2_to_linear (*src++);
       *dst++ = *src++;
     }
-
-  return total;
 }
 
-static inline long
-conv_yF_linear_yF_gamma (const float *src, float *dst, long samples)
+static inline void
+conv_yF_linear_yF_gamma (const Babl *conversion,const float *src, float *dst, long samples)
 {
-  long total = samples;
-
   const __v4sf *s = (const __v4sf*)src;
         __v4sf *d = (__v4sf*)dst;
 
@@ -490,15 +472,11 @@ conv_yF_linear_yF_gamma (const float *src, float *dst, long samples)
     {
       *dst++ = babl_linear_to_gamma_2_2 (*src++);
     }
-
-  return total;
 }
 
-static inline long
-conv_yF_gamma_yF_linear (const float *src, float *dst, long samples)
+static inline void
+conv_yF_gamma_yF_linear (const Babl *conversion,const float *src, float *dst, long samples)
 {
-  long total = samples;
-
   const __v4sf *s = (const __v4sf*)src;
         __v4sf *d = (__v4sf*)dst;
 
@@ -530,22 +508,20 @@ conv_yF_gamma_yF_linear (const float *src, float *dst, long samples)
     {
       *dst++ = babl_gamma_2_2_to_linear (*src++);
     }
-
-  return total;
 }
 
 
-static long
-conv_rgbF_linear_rgbF_gamma (const float *src, float *dst, long samples)
+static void
+conv_rgbF_linear_rgbF_gamma (const Babl *conversion,const float *src, float *dst, long samples)
 {
-  return conv_yF_linear_yF_gamma (src, dst, samples * 3) / 3;
+  conv_yF_linear_yF_gamma (conversion, src, dst, samples * 3);
 }
 
 
-static long
-conv_rgbF_gamma_rgbF_linear (const float *src, float *dst, long samples)
+static void
+conv_rgbF_gamma_rgbF_linear (const Babl *conversion,const float *src, float *dst, long samples)
 {
-  return conv_yF_gamma_yF_linear (src, dst, samples * 3) / 3;
+  conv_yF_gamma_yF_linear (conversion, src, dst, samples * 3);
 }
 
 #endif /* defined(USE_SSE2) */
